return {
    monthsDuration = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
    daysInWeek = 7,
    startingYear = 427,
    startingYearDay = 227,
    -- Friday, calculated from TES2 and TES4's starting week days (see #7546)
    startingWeekDay = 5,
}
