---
-- @context global|menu|local|player
-- @module interfaces
-- @usage local I = require('openmw.interfaces')

---
-- @field [parent=#interfaces] scripts.omw.activationhandlers#scripts.omw.activationhandlers Activation

---
-- @field [parent=#interfaces] scripts.omw.mechanics.animationcontroller#scripts.omw.mechanics.animationcontroller AnimationController

---
-- @field [parent=#interfaces] scripts.omw.ai#scripts.omw.ai AI

---
-- @field [parent=#interfaces] scripts.omw.camera.camera#scripts.omw.camera.camera Camera

---
-- @field [parent=#interfaces] scripts.omw.combat.local#scripts.omw.combat.local Combat

---
-- @field [parent=#interfaces] scripts.omw.mwui.init#scripts.omw.mwui.init MWUI

---
-- @field [parent=#interfaces] scripts.omw.settings.player#scripts.omw.settings.player Settings

---
-- @field [parent=#interfaces] scripts.omw.ui#scripts.omw.ui UI

---
-- @field [parent=#interfaces] scripts.omw.usehandlers#scripts.omw.usehandlers ItemUsage

---
-- @field [parent=#interfaces] scripts.omw.skillhandlers#scripts.omw.skillhandlers SkillProgression

---
-- @field [parent=#interfaces] scripts.omw.crimes#scripts.omw.crimes Crimes

---
-- @function [parent=#interfaces] __index
-- @param #interfaces self
-- @param #string key
-- @return #any

return nil
