#ifndef LIB_SKY_PASSES
#define LIB_SKY_PASSES

#define PASS_ATMOSPHERE 0
#define PASS_ATMOSPHERE_NIGHT 1
#define PASS_CLOUDS 2
#define PASS_MOON 3
#define PASS_SUN 4
#define PASS_SUNFLASH_QUERY 5
#define PASS_SUNGLARE 6

#endif
