return {
    monthsDuration = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
    daysInWeek = 7,

    -- It is the day of OpenMW 0.1.0 release, we use it as default starting date
    startingYear = 2008,
    startingYearDay = 151,
    startingWeekDay = 0,
}
